@@ -1,6 +1,6 @@
 __author__ = "Joseph D. Hughes"
-__date__ = "Jan 19, 2023"
-__version__ = "0.1.5"
+__date__ = "Jul 26, 2023"
+__version__ = "0.2.0"
 __maintainer__ = "Joseph D. Hughes"
 __email__ = "jdhughes@usgs.gov"
 __status__ = "Production"
