@@ -2,7 +2,7 @@ from collections import OrderedDict
 from itertools import groupby
 from os import PathLike, environ
 from pathlib import Path
-from shutil import copytree
+from shutil import copytree, rmtree
 from typing import Dict, List, Optional
 
 import pytest
@@ -23,11 +23,17 @@ def function_tmpdir(tmpdir_factory, request) -> Path:
 
     keep = request.config.getoption("--keep")
     if keep:
-        copytree(temp, Path(keep) / temp.name)
+        path = Path(keep) / temp.name
+        if path.is_dir():
+            rmtree(path)
+        copytree(temp, path)
 
     keep_failed = request.config.getoption("--keep-failed")
     if keep_failed and request.node.rep_call.failed:
-        copytree(temp, Path(keep_failed) / temp.name)
+        path = Path(keep_failed) / temp.name
+        if path.is_dir():
+            rmtree(path)
+        copytree(temp, path)
 
 
 @pytest.fixture(scope="class")
@@ -40,7 +46,10 @@ def class_tmpdir(tmpdir_factory, request) -> Path:
 
     keep = request.config.getoption("--keep")
     if keep:
-        copytree(temp, Path(keep) / temp.name)
+        path = Path(keep) / temp.name
+        if path.is_dir():
+            rmtree(path)
+        copytree(temp, path)
 
 
 @pytest.fixture(scope="module")
@@ -50,8 +59,10 @@ def module_tmpdir(tmpdir_factory, request) -> Path:
 
     keep = request.config.getoption("--keep")
     if keep:
-        copytree(temp, Path(keep) / temp.name)
-        print(list((Path(keep) / temp.name).rglob("*")))
+        path = Path(keep) / temp.name
+        if path.is_dir():
+            rmtree(path)
+        copytree(temp, path)
 
 
 @pytest.fixture(scope="session")
@@ -61,7 +72,10 @@ def session_tmpdir(tmpdir_factory, request) -> Path:
 
     keep = request.config.getoption("--keep")
     if keep:
-        copytree(temp, Path(keep) / temp.name)
+        path = Path(keep) / temp.name
+        if path.is_dir():
+            rmtree(path)
+        copytree(temp, path)
 
 
 # environment-dependent fixtures
@@ -176,15 +190,33 @@ def pytest_generate_tests(metafunc):
         repos_path = Path(repos_path)
     else:
         # by default, assume external repositories are
-        # in the same directory as the current project
-        # and tests are run from <proj root>/autotest.
-        # if no models are found, tests requesting the
-        # fixtures will be skipped.
+        # level (side-by-side) on the filesystem with
+        # the consuming project. also assume tests are
+        # run from <proj root>/autotest.
+        #
+        # external model repo directories are expected
+        # to be named identically to the GitHub repos,
+        # e.g. "modflow6-testmodels", with an optional
+        # ".git" suffix appended to the directory name.
+        #
+        # if model repositories are not found in either
+        # the default location or in REPOS_PATH, tests
+        # requesting these fixtures will be skipped.
         repos_path = Path.cwd().parent.parent
 
+    def get_repo_path(repo_name: str) -> Optional[Path]:
+        """Get the path for the repository with the given name
+        (optionally with .git suffix), or None if not found"""
+        repo_path = repos_path / repo_name
+        if not repo_path.is_dir():
+            repo_path = repos_path / (repo_name + ".git")
+        if not repo_path.is_dir():
+            repo_path = None
+        return repo_path
+
     key = "test_model_mf6"
     if key in metafunc.fixturenames:
-        repo_path = repos_path / "modflow6-testmodels"
+        repo_path = get_repo_path("modflow6-testmodels")
         namefile_paths = (
             get_namefile_paths(
                 repo_path / "mf6",
@@ -193,7 +225,7 @@ def pytest_generate_tests(metafunc):
                 selected=models_selected,
                 packages=packages_selected,
             )
-            if repo_path.is_dir()
+            if repo_path
             else []
         )
         metafunc.parametrize(
@@ -202,7 +234,7 @@ def pytest_generate_tests(metafunc):
 
     key = "test_model_mf5to6"
     if key in metafunc.fixturenames:
-        repo_path = repos_path / "modflow6-testmodels"
+        repo_path = get_repo_path("modflow6-testmodels")
         namefile_paths = (
             get_namefile_paths(
                 repo_path / "mf5to6",
@@ -212,7 +244,7 @@ def pytest_generate_tests(metafunc):
                 selected=models_selected,
                 packages=packages_selected,
             )
-            if repo_path.is_dir()
+            if repo_path
             else []
         )
         metafunc.parametrize(
@@ -221,7 +253,7 @@ def pytest_generate_tests(metafunc):
 
     key = "large_test_model"
     if key in metafunc.fixturenames:
-        repo_path = repos_path / "modflow6-largetestmodels"
+        repo_path = get_repo_path("modflow6-largetestmodels")
         namefile_paths = (
             get_namefile_paths(
                 repo_path,
@@ -231,7 +263,7 @@ def pytest_generate_tests(metafunc):
                 selected=models_selected,
                 packages=packages_selected,
             )
-            if repo_path.is_dir()
+            if repo_path
             else []
         )
         metafunc.parametrize(
@@ -240,7 +272,7 @@ def pytest_generate_tests(metafunc):
 
     key = "example_scenario"
     if key in metafunc.fixturenames:
-        repo_path = repos_path / "modflow6-examples"
+        repo_path = get_repo_path("modflow6-examples")
 
         def is_nested(namfile_path: PathLike) -> bool:
             p = Path(namfile_path)
@@ -277,7 +309,12 @@ def pytest_generate_tests(metafunc):
 
         def get_examples():
             # find MODFLOW 6 namfiles
-            namfiles = [p for p in (repo_path / "examples").rglob("mfsim.nam")]
+            examples_path = repo_path / "examples"
+            namfiles = (
+                [p for p in examples_path.rglob("mfsim.nam")]
+                if examples_path.is_dir()
+                else []
+            )
 
             # group by scenario
             examples = group_examples(namfiles)
@@ -320,7 +357,7 @@ def pytest_generate_tests(metafunc):
 
             return examples
 
-        example_scenarios = get_examples() if repo_path.is_dir() else dict()
+        example_scenarios = get_examples() if repo_path else dict()
         metafunc.parametrize(
             key,
             [(name, nfps) for name, nfps in example_scenarios.items()],
